@@ -49,7 +49,7 @@ def xiami_download_song(sid, output_dir = '.', merge = True, info_only = False):
     i = doc.getElementsByTagName("track")[0]
     artist = i.getElementsByTagName("artist")[0].firstChild.nodeValue
     album_name = i.getElementsByTagName("album_name")[0].firstChild.nodeValue
-    song_title = i.getElementsByTagName("title")[0].firstChild.nodeValue
+    song_title = i.getElementsByTagName("name")[0].firstChild.nodeValue
     url = location_dec(i.getElementsByTagName("location")[0].firstChild.nodeValue)
     try:
         lrc_url = i.getElementsByTagName("lyric")[0].firstChild.nodeValue
@@ -152,7 +152,10 @@ def xiami_download(url, output_dir = '.', stream_type = None, merge = True, info
         id = r1(r'http://www.xiami.com/collect/(\d+)', url)
         xiami_download_showcollect(id, output_dir, merge, info_only)
 
-    if re.match('http://www.xiami.com/song/\d+', url):
+    if re.match(r'http://www.xiami.com/song/\d+\b', url):
+        id = r1(r'http://www.xiami.com/song/(\d+)', url)
+        xiami_download_song(id, output_dir, merge, info_only)
+    elif re.match(r'http://www.xiami.com/song/\w+', url):
         html = get_html(url, faker=True)
         id = r1(r'rel="canonical" href="http://www.xiami.com/song/([^"]+)"', html)
         xiami_download_song(id, output_dir, merge, info_only)
